package auth

import (
	"context"
	"fmt"
	"net/http"

	"github.com/Sirupsen/logrus"
)

// Middleware get User from session and put it in context
type Middleware struct {
	authorizer *Authorizer
	log        *logrus.Entry
}

// NewMiddleware returns a new authentication middleware
func NewMiddleware(authorizer *Authorizer, log *logrus.Entry) *Middleware {
	return &Middleware{
		authorizer: authorizer,
		log:        log.WithField("middleware", "auth"),
	}
}

func (m *Middleware) ServeHTTP(w http.ResponseWriter, r *http.Request, next http.HandlerFunc) {
	user, err := m.authorizer.CurrentUser(w, r)
	if err != nil {
		panic(err)
	}

	m.log.Debug("setting user in the context")
	ctx := context.WithValue(r.Context(), "auth.user", user)
	r = r.WithContext(ctx)

	next(w, r)
}

// MiddlewareRole handles the role checking for the current user
type MiddlewareRole struct {
	authorizer      *Authorizer
	log             *logrus.Entry
	role            string
	loginPageGetter func() string
}

// NewMiddlewareRole returns a new MiddlewareRole
func NewMiddlewareRole(authorizer *Authorizer, log *logrus.Entry, loginPageGetter func() string, role string) *MiddlewareRole {
	return &MiddlewareRole{
		authorizer:      authorizer,
		log:             log.WithField("middleware", "role"),
		role:            role,
		loginPageGetter: loginPageGetter,
	}
}

func (m *MiddlewareRole) ServeHTTP(w http.ResponseWriter, r *http.Request, next http.HandlerFunc) {
	user := GetCurrentUser(r, m.log)

	if user == nil || !user.HasRole(m.role) {
		if user == nil {
			m.log.Debug("user is nil in the context")
		} else {
			m.log.Debug("user doesn't have the role")
		}

		cookie, err := m.authorizer.Cookiejar.Get(r, "rlogin")
		if err != nil {
			panic(err)
		}
		cookie.Values["redirect"] = r.URL.Path
		err = cookie.Save(r, w)
		if err != nil {
			panic(err)
		}
		http.Redirect(w, r, m.loginPageGetter(), http.StatusTemporaryRedirect)
		return
	}

	m.log.Debug("user has the role, continuing")

	next(w, r)
}

// GetPostLoginRedirect returns the location of the page requested before the
// users was redirected to the login page
func GetPostLoginRedirect(a *Authorizer, w http.ResponseWriter, r *http.Request) (string, error) {
	cookie, err := a.Cookiejar.Get(r, "rlogin")
	if err != nil {
		return "", err
	}
	val := cookie.Values["redirect"]
	if val == nil {
		return "", nil
	}
	path, ok := val.(string)
	if !ok {
		return "", fmt.Errorf("invalid redirect type")
	}
	cookie.Values["rlogin"] = ""
	err = cookie.Save(r, w)
	if err != nil {
		return "", err
	}
	return path, nil

}

// GetCurrentUser gets the current user from the request context
func GetCurrentUser(r *http.Request, log *logrus.Entry) User {
	log.Debug("getting user from context")

	u := r.Context().Value("auth.user")
	if u == nil {
		return nil
	}
	user, ok := u.(User)
	if !ok {
		panic("invalid user type")
	}
	return user
}
