package main

import (
	"net/http"
	"os"

	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/auth"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/config"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/movies"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/users"
	"gitlab.quimbo.fr/odwrtw/canape-sql/src/internal/web"

	"github.com/Sirupsen/logrus"
	"github.com/codegangsta/negroni"
	"github.com/gorilla/sessions"
	"github.com/jmoiron/sqlx"
	_ "github.com/lib/pq"
)

// UserBackend represents the data backend to get the user
type UserBackend struct {
	Database *sqlx.DB
}

// Get gets the username from the UserBackend
func (b *UserBackend) Get(username string) (auth.User, error) {
	return users.Get(b.Database, username)
}

func main() {
	var cfgPath string
	cfgPath = os.Getenv("CONFIG_FILE")
	if cfgPath == "" {
		cfgPath = "./config.yml"
	}

	// Setup the logger
	logger := logrus.New()
	logger.Formatter = &logrus.TextFormatter{FullTimestamp: true}
	logger.Level = logrus.DebugLevel

	log := logrus.NewEntry(logger)
	cf, err := config.Load(cfgPath)
	if err != nil {
		log.Panic(err)
	}

	db, err := sqlx.Connect("postgres", cf.PGDSN)
	if err != nil {
		log.Panic(err)
	}
	uBackend := &UserBackend{Database: db}

	authParams := auth.Params{
		Backend:    uBackend,
		Pepper:     cf.Authorizer.Pepper,
		CookieName: cf.Authorizer.CookieName,
		Cookiejar:  sessions.NewCookieStore([]byte(cf.Authorizer.Key)),
		Cost:       cf.Authorizer.Cost,
	}
	authorizer := auth.New(authParams)

	env := web.NewEnv(web.EnvParams{
		Database: db,
		Auth:     authorizer,
		Log:      log,
		Config:   cf,
	})

	authMiddleware := auth.NewMiddleware(env.Auth, log)

	env.Handle("/users/login", users.LoginGETHandler).Name("users.login").Methods("GET")
	env.Handle("/users/login", users.LoginPOSTHandler).Name("users.login").Methods("POST")
	env.Handle("/users/logout", users.LogoutHandler).Name("users.logout")
	env.Handle("/users/details", users.DetailsHandler).Name("users.details").WithRole(users.UserRole)
	env.Handle("/users/edit", users.EditHandler).Name("users.edit").WithRole(users.UserRole)

	env.Handle("/movies/polochon", movies.FromPolochon).Name("movies.polochon").WithRole(users.UserRole)
	env.Handle("/movies/explore/popular", movies.ExplorePopular).Name("movies.explore.popular").WithRole(users.UserRole)

	err = env.SetLoginRoute("users.login")
	if err != nil {
		log.Panic(err)
	}

	n := negroni.Classic()
	n.Use(authMiddleware)
	n.Use(negroni.NewStatic(http.Dir(cf.PublicDir)))
	n.UseHandler(env.Router)
	n.Run(":" + cf.Port)
}
